from csp import Constraint, CSP
from typing import Dict, List, Optional


class MapColoringConstraint(Constraint[str, str]):
    def __init__(self, place1: str, place2: str) -> None:
        super().__init__([place1, place2])
        self.place1: str = place1
        self.place2: str = place2

    def satisfied(self, assignment: Dict[str, str]) -> bool:
        # если какой-либо регион place отсутствует в присваивании,
        # то его цвета не могут привести к конфликту
        if self.place1 not in assignment or self.place2 not in assignment:
            return True
        # проверяем, не совпадает ли цвет, присвоенный pLace1,
        # с цветом, присвоенным place2
        return assignment[self.place1] != assignment[self.place2]


if __name__ == "__main__":
    variables: List[str] = ["Западная Австралия", "Северная территория", "Южная Австралия",
                            "Квинсленд", "Новый Южный Уэльс", "Виктория", "Тасмания"]
    domains: Dict[str, List[str]] = {}
    for variable in variables:
        domains[variable] = ["красный", "зеленый", "синий"]
    csp: CSP[str, str] = CSP(variables, domains)
    csp.add_constraint(MapColoringConstraint("Западная Австралия", "Северная территория"))
    csp.add_constraint(MapColoringConstraint("Западная Австралия", "Южная Австралия"))
    csp.add_constraint(MapColoringConstraint("Южная Австралия", "Северная территория"))
    csp.add_constraint(MapColoringConstraint("Квинсленд", "Северная территория"))
    csp.add_constraint(MapColoringConstraint("Квинсленд", "Южная Австралия"))
    csp.add_constraint(MapColoringConstraint("Квинсленд", "Новый Южный Уэльс"))
    csp.add_constraint(MapColoringConstraint("Новый Южный Уэльс", "Южная Австралия"))
    csp.add_constraint(MapColoringConstraint("Виктория", "Южная Австралия"))
    csp.add_constraint(MapColoringConstraint("Виктория", "Новый Южный Уэльс"))
    csp.add_constraint(MapColoringConstraint("Виктория", "Тасмания"))

    solution: Optional[Dict[str, str]] = csp.backtracking_search()
    if solution is None:
        print("Решение не найдено!")
    else:
        print(solution)
